/*
	CreateSpatialTemplateCmd.h - make spatial parameter template files for wave

	Will 18, 2001, 2006: will@opaline.demon.co.uk
*/

#ifndef CREATESPATIALTEMPLATECMD_H
#define CREATESPATIALTEMPLATECMD_H

#include <NoUndoCmd.h>
#include <GraphFltCmd.h>
#include "NewSpatialTemplateCmd.h"


typedef void ( *CreateSpatialTemplateCallback ) ( class CreateSpatialTemplateCmd *, void *, unsigned int );


class GraphIntCmd;
class GraphFltCmd;
class DMenuCmd;
class LineEdit;
class CreateSpatialScoreCmd;
class RunCsoundCmd;
class ShapeSpatialPakTemplateCmd;
class RawToWavCmd;


class CreateSpatialTemplateCmd : public NoUndoCmd
{
	protected:
		virtual void doCommand();	// CreateSpatialTemplate
		CreateSpatialTemplateCallback _callback;
		void * _clientData;
		const char *_name;

	public:
		CreateSpatialTemplateCmd ( const char *, int active, unsigned int, SpatialPak *, CreateSpatialTemplateCallback, void * );
		~CreateSpatialTemplateCmd();
		virtual const char *const className() { return ( "CreateSpatialTemplateCmd" ); }

	private:
		static void menuSelectionCallback ( void *, unsigned int, char * );
		static void pathMenuSelectionCallback ( void *, unsigned int, char * );
		static void paramRangeMenuSelectionCallback ( void *, unsigned int, char * );
		static void waveViewCallback ( void *, void *, long, long, long,
											long, long, long, Boolean, unsigned, unsigned );
		static void waveDataCmdCallback ( void *, void *, long, long, long,
											long, long, long, Boolean, unsigned, unsigned );
		void menuSelection ( char *, unsigned int );
		void waveViewCallbackFinished ( GraphIntCmd *, long, long, long, long, long,
											long, Boolean, unsigned, unsigned );

		static void viewReverbTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewReverbTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );


		static void viewRangeTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewRangeTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewBwidthTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewBwidthTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewSweepTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewSweepTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewPanTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewPanTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewDopplerTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewDopplerTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewReverbLTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewReverbLTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewReverbRTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewReverbRTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewPanLTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewPanLTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewPanRTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewPanRTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewBwidthLTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewBwidthLTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewBwidthRTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewBwidthRTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewSweepLTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewSweepLTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewSweepRTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewSweepRTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewAmpLTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewAmpLTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void viewAmpRTemplateCmdCallback ( void *, void *, long, long, long,
												long, long, float, Boolean, unsigned, unsigned );

		void viewAmpRTemplateCmdCallbackFinished ( GraphIntCmd *, GraphFltCmd *, long, long, long, long, long, long,
												float, Boolean, unsigned, unsigned );

		static void panTypeEnteredCallback ( void *, char * );
		static void panTypeCancelledCallback ( void * );
		void panTypeEntered ( char * );

		static void filterTypeEnteredCallback ( void *, char * );
		static void filterTypeCancelledCallback ( void * );
		void filterTypeEntered ( char * );

		static void angularSepEnteredCallback ( void *, char * );
		static void angularSepCancelledCallback ( void * );
		void angularSepEntered ( char * );

		static void signalScaleEnteredCallback ( void *, char * );
		static void signalScaleCancelledCallback ( void * );
		void signalScaleEntered ( char * );

		static void detuneEnteredCallback ( void *, char * );
		static void detuneCancelledCallback ( void * );
		void detuneEntered ( char * );

		static void phaserEnteredCallback ( void *, char * );
		static void phaserCancelledCallback ( void * );
		void phaserEntered ( char * );

		static void rvbTailEnteredCallback ( void *, char * );
		static void rvbTailCancelledCallback ( void * );
		void rvbTailEntered ( char * );

		static void panLTypeEnteredCallback ( void *, char * );
		static void panLTypeCancelledCallback ( void * );
		void panLTypeEntered ( char * );

		static void panRTypeEnteredCallback ( void *, char * );
		static void panRTypeCancelledCallback ( void * );
		void panRTypeEntered ( char * );

		static void delayLEnteredCallback ( void *, char * );
		static void delayLCancelledCallback ( void * );
		void delayLEntered ( char * );

		static void delayREnteredCallback ( void *, char * );
		static void delayRCancelledCallback ( void * );
		void delayREntered ( char * );

		static void filterLTypeEnteredCallback ( void *, char * );
		static void filterLTypeCancelledCallback ( void * );
		void filterLTypeEntered ( char * );

		static void filterRTypeEnteredCallback ( void *, char * );
		static void filterRTypeCancelledCallback ( void * );
		void filterRTypeEntered ( char * );

		static void rvbRangeEnteredCallback ( void *, char * );
		static void rvbRangeCancelledCallback ( void * );
		void rvbRangeEntered ( char * );

		static void distRangeEnteredCallback ( void *, char * );
		static void distRangeCancelledCallback ( void * );
		void distRangeEntered ( char * );

		static void bwRangeEnteredCallback ( void *, char * );
		static void bwRangeCancelledCallback ( void * );
		void bwRangeEntered ( char * );

		static void sweepRangeEnteredCallback ( void *, char * );
		static void sweepRangeCancelledCallback ( void * );
		void sweepRangeEntered ( char * );

		static void dopplerRangeEnteredCallback ( void *, char * );
		static void dopplerRangeCancelledCallback ( void * );
		void dopplerRangeEntered ( char * );

		static void rvbLRangeEnteredCallback ( void *, char * );
		static void rvbLRangeCancelledCallback ( void * );
		void rvbLRangeEntered ( char * );

		static void rvbRRangeEnteredCallback ( void *, char * );
		static void rvbRRangeCancelledCallback ( void * );
		void rvbRRangeEntered ( char * );

		static void bwLRangeEnteredCallback ( void *, char * );
		static void bwLRangeCancelledCallback ( void * );
		void bwLRangeEntered ( char * );

		static void bwRRangeEnteredCallback ( void *, char * );
		static void bwRRangeCancelledCallback ( void * );
		void bwRRangeEntered ( char * );

		static void sweepLRangeEnteredCallback ( void *, char * );
		static void sweepLRangeCancelledCallback ( void * );
		void sweepLRangeEntered ( char * );

		static void sweepRRangeEnteredCallback ( void *, char * );
		static void sweepRRangeCancelledCallback ( void * );
		void sweepRRangeEntered ( char * );

		static void ampLRangeEnteredCallback ( void *, char * );
		static void ampLRangeCancelledCallback ( void * );
		void ampLRangeEntered ( char * );

		static void ampRRangeEnteredCallback ( void *, char * );
		static void ampRRangeCancelledCallback ( void * );
		void ampRRangeEntered ( char * );

		static void spatialScoreCreatedCallback ( BreakCmd *, Boolean, void * );
		void createSpatialScore ( void );
		void spatialScoreCreated ( CreateSpatialScoreCmd * );

		static void csoundRunCallback ( BreakCmd *, Boolean, void * );
		void csoundRun ( RunCsoundCmd * );

		static void rawToWavFinishedCallback ( BreakCmd *, Boolean, void * );
		void rawToWav ( char * );
		void rawToWavFinished ( RawToWavCmd * );

		void createWaveDataArray ( unsigned, long );
		float *createFloatDataArray ( char *, float *, long, long );

		void minmax ( long, long, long ,long );
		void exit ( void );
		void initError ( char *, char * );
		void deleteMenuObjects();
		void deletePathMenuObjects();
		void deleteParamsRangeMenuObjects();
		
		long _currentSamples;
		unsigned _arrayItems;	// num items on menu
		Boolean _viewRealised;
		Boolean _menuRealised;
		Boolean _pathMenuRealised;
		Boolean _paramRangeMenuRealised;
		Boolean _rvbViewRealised;
		Boolean _rangeViewRealised;
		Boolean _bwViewRealised;
		Boolean _sweepViewRealised;
		Boolean _panViewRealised;
		Boolean _dopplerViewRealised;
		Boolean _rvbLViewRealised;
		Boolean _rvbRViewRealised;		
		Boolean _panLViewRealised;		
		Boolean _panRViewRealised;		
		Boolean _bwLViewRealised;		
		Boolean _bwRViewRealised;		
		Boolean _sweepLViewRealised;		
		Boolean _sweepRViewRealised;		
		Boolean _ampLViewRealised;		
		Boolean _ampRViewRealised;
		Boolean _panTypeEditRealised;
		Boolean _filterTypeEditRealised;
		Boolean _angularSepEditRealised;
		Boolean _signalScaleEditRealised;
		Boolean _detuneEditRealised;
		Boolean _phaserEditRealised;
		Boolean _rvbTailEditRealised;
		Boolean _panLTypeEditRealised;		
		Boolean _panRTypeEditRealised;
		Boolean _delayLEditRealised;		
		Boolean _delayREditRealised;
		Boolean _filterLTypeEditRealised;		
		Boolean _filterRTypeEditRealised;
		Boolean _rvbRangeEditRealised;
		Boolean _distRangeEditRealised;
		Boolean _bwRangeEditRealised;
		Boolean _sweepRangeEditRealised;
		Boolean _dopplerRangeEditRealised;
		Boolean _rvblRangeEditRealised;
		Boolean _rvbrRangeEditRealised;
		Boolean _bwlRangeEditRealised;
		Boolean _bwrRangeEditRealised;
		Boolean _sweeplRangeEditRealised;
		Boolean _sweeprRangeEditRealised;
		Boolean _amplRangeEditRealised;
		Boolean _amprRangeEditRealised;
		Boolean _shapeCmdRealised;
		unsigned int _error;
		unsigned int _waveNum;
		unsigned int _bytesPerSample;
		SpatialPak *_spatialPak;
		/* storage for various file names */
		char _rvbFileName[BUFSIZE];
		char _rangeFileName[BUFSIZE];
		char _bwFileName[BUFSIZE];
		char _sweepFileName[BUFSIZE];
		char _panFileName[BUFSIZE];
		char _dplrFileName[BUFSIZE];
		char _rvbLFileName[BUFSIZE];
		char _rvbRFileName[BUFSIZE];
		char _panLFileName[BUFSIZE];
		char _panRFileName[BUFSIZE];
		char _bwLFileName[BUFSIZE];
		char _bwRFileName[BUFSIZE];
		char _sweepLFileName[BUFSIZE];
		char _sweepRFileName[BUFSIZE];
		char _ampLFileName[BUFSIZE];
		char _ampRFileName[BUFSIZE];
		DMenuCmd *_menu, *_pathMenu, *_paramRangeMenu;
		ShapeSpatialPakTemplateCmd *_shapeCmd;
		LineEdit *_panTypeEdit, *_filterTypeEdit, *_angularSepEdit;
		LineEdit *_signalScaleEdit, *_detuneEdit, *_phaserEdit, *_rvbTailEdit;
		LineEdit *_panLTypeEdit, *_panRTypeEdit, *_delayLEdit, *_delayREdit;
		LineEdit *_filterLTypeEdit, *_filterRTypeEdit, *_rvbRangeEdit;
		LineEdit * _distRangeEdit, *_bwRangeEdit, *_sweepRangeEdit;
		LineEdit *_dopplerRangeEdit, *_rvblRangeEdit, *_rvbrRangeEdit;
		LineEdit *_bwlRangeEdit, *_bwrRangeEdit, *_sweeplRangeEdit, *_sweeprRangeEdit;
		LineEdit *_amplRangeEdit, *_amprRangeEdit;
		long _min, _max;
		float _rvbScale, _rangeScale, _bwScale, _sweepScale, _panScale;
		float _dplrScale, _rvblScale, _rvbrScale, _panlScale, _panrScale;;
		float _bwlScale, _bwrScale, _sweeplScale, _sweeprScale, _amplScale,	_amprScale;
		CreateSpatialScoreCmd *_cssCmd;
		RunCsoundCmd *_runCsndCmd;
};

#endif
