/*
    MixerHelpCmd.cc

    Will Eighteen, 2000, 2006: will@opaline.demon.co.uk
*/

#include "MixerHelpCmd.h"
#include "Application.h"
#include <TextEdit.h>


MixerHelpCmd::MixerHelpCmd ( const char *name, int active ) :  NoUndoCmd ( name, active )
{
	_editRealised = FALSE;
}


MixerHelpCmd::~MixerHelpCmd()
{
	// empty
}


void MixerHelpCmd::doCommand()
{
	if (_editRealised == TRUE )
		return;


	_edit = new TextEdit ( ( char * ) name(), TRUE, &MixerHelpCmd::okCallback, &MixerHelpCmd::cancelCallback, ( void * ) this );	
	_edit->setRows ( 19 );
	_edit->setColumns ( 70 );
	_editRealised = TRUE;
	_edit->execute();
	_edit->setText ( "The mixer consists of two parts, 'New / Mix' to set the \
mixdown parameters (gain and pan) and 'Mixdown' which produces the mixed \
output file using the supplied parameters.\n\nMixdown parameters\n'New / Mix' \
produces four popup windows:\n\n1) wave select menu;\n2) mix display \
panel;\n3) gain menu;\n4) pan menu.\n\nThe mix can be statically \
controlled, as on a conventional physical desk, with constant settings of \
gain and pan for a particular track, or dynamically, having the \
effect of being able to continuously vary gain and pan on a per-track basis \
whilst the mixdown is in progress, only possible to a very limited extent \
on the real physical mixing desk.\n\n\n1) The wave select menu.\nThis pops up \
a menu of buttons with the names of the currently loaded tracks. Note that \
this menu, in common with other wave select menus, is static, i.e. constructed \
on the basis of files loaded when it was popped up. If a wave (or template) is \
loaded after 'New Mix' is activated it will not be shown in any of the mixer \
menus but the sound track _will_ be included in the mixdown. Always use \
'New / Mix' after loading all the tracks or templates required, or close it \
down and restart it.\nIn order to set the gain and pan for a track, the \
appropriate button bearing the name of the file must be left-clicked to \
select it: the name then appears at the top of the gain and pan menus, \
allowing these parameters to be set for that track.\nThe 'Quit'button is a \
global 'Quit' for the mix control panels. All of them will be popped down \
if this item is selected.\n\n\n2) The mix display pane.\nThis displays a list \
of currently loaded waves. By default, if no mix settings \
have been chosen, the name of each wave is shown followed by default constant \
gain and pan settings: unity gain (x1) and 0.5 (pan central) (these default \
values are set in Options / Mix / Mix parameters). The gain and pan for a \
given track are set by first selecting the wave to which the gain and pan \
apply, as above, thus activating the gain and pan menus for the selected track, \
and then choosing appropriate gain and pan values.\n\nBy default, the gain \
and pan menus have two buttons in their popups: 'Global value' and 'quit'. If \
templates are loaded as well as wave files, these will appear as labelled \
buttons before the 'Global value' and 'Quit' buttons.\nLoaded templates \
appear in both gain and pan menus; if one is used for, say, gain, it is \
then unavailable for use with 'pan'. Templates cannot be shared between \
tracks.\n\nA template is a file full of numbers \
which represents the motion of either the gain fader or the pan potentiometer \
for a track. Only floating-point templates are used in the mixer. Typically \
they will be constructed either by hand-drawing in the template view pane or \
by selecting the whole template and using 'Edit / Shape template'.\nSince \
template files can be used to represent position of a control with time, \
allocation of a template to gain and pan for tracks enables their continuous \
variation during mixdown; this is equivalent to having as many pairs of \
hands on the desk as tracks being mixed. It also in effect means that the \
control position variations are recorded. Since the whole mix setup can be \
saved with 'State / Save' this enables reconstruction of any given mix by \
loading its state file.\n\nNote that the 'Quit' button in the mix display \
panel only closes this window. The wave select, gain and pan menus continue \
to be available, i.e. mix parameters can be set without the mix display \
active. If the mix display pane has been closed, it can only be re-activated \
by closing the mix with 'Quit' in the wave select menu, then re-selecting \
'New / Mix'.\n\n\n3) The gain menu\nInitially, with no wave selected, this \
popup contains two buttons 'Global value' and 'Quit' with the heading 'Wave: \
none' at the top. If templates are also loaded, they appear as supplementary \
buttons. The buttons other than 'Quit' can only be activated if a wave has \
been selected with the wave select menu. The name of the selected wave then \
appears at the top of the gain popup window.\nIf 'Global value' is selected \
a text entry popup appears, into which the desired gain value can be entered; \
'Return' should be used to confirm the value. The popup will disappear and \
the chosen gain value for the track will be shown in the mix display pane \
preceded by 'global' to show it is a static value (in fact it is still a \
template, but one whose datum points all have the same value).\nIf, instead \
of a static gain a template is selected, then its name appears as the pan \
template for the selected wave in the mix display pane. The pan is then \
dynamically controlled by this template during mixdown.\n\n\n4) \
The pan menu\nThis works in exactly the same way as the gain menu, except that \
where the gain has a default constant value of unity, the pan has a default \
constant value of 0.5 (pan central).\nNote that the default pan and gain are \
defined in Options / Mix / Mix parameters.\n\nWhen the mix has been set up \
as desired, the 'Mixdown' button on the 'File' menu causes loaded waves to be \
mixed down using the gain and pan control values in the mix display pane. \
Note that the mix also depends on the settings of Options / System control \
and Options / Wave control. The format of the output wave file is determined \
by Options / File / Wave parameters, with the exception that the 'channels' \
setting is ignored and a value of 2 (stereo) assumed by the mixer.");

	_edit->setEditStatus ( False );
}


void MixerHelpCmd::okCallback ( XtPointer clientData, char * text )
{
	MixerHelpCmd *obj = ( MixerHelpCmd * ) clientData;

	obj->ok( clientData );	// ok mapped to 'enter'
}


void MixerHelpCmd::cancelCallback ( XtPointer clientData )
{
	MixerHelpCmd *obj = ( MixerHelpCmd * ) clientData;

	obj->ok( clientData );	// call ok to delete object
}


void MixerHelpCmd::ok ( void *clientData )
{
	MixerHelpCmd *obj = ( MixerHelpCmd * ) clientData;

	if (obj->_editRealised == TRUE )
	{
		obj->_editRealised = FALSE;
		delete obj->_edit;
	}
}
