/*
    WaveDefaultsCmd.cc
 
    Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include <stdio.h>
#include "WaveDefaultsCmd.h"
#include "Application.h"
#include <LineEdit.h>
#include <InfoDialogManager.h>
#include "OpResources.h"	// this is the only resources file with global resources


WaveDefaultsCmd::WaveDefaultsCmd ( const char *name, int active ) :  Cmd ( name, active )
{
	_popupRealised = False;
}


void WaveDefaultsCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	char buf[256];
	char help[320];
		
	_edit = new LineEdit ( ( char * ) name(), TRUE,
									&WaveDefaultsCmd::enterCallback,
									&WaveDefaultsCmd::cancelCallback, ( void * ) this );	
                
	/* retrieve resource to be displayed */
 	snprintf ( buf, sizeof ( buf ), "%d,%ld,%d,%ld",mixwaveResources->bitsPerSample(),
				mixwaveResources->samplesPerSec(),
				mixwaveResources->channels(),
				mixwaveResources->defaultWavNumSamples() );

	buf [ sizeof ( buf ) - 1 ] = 0;		
	_edit->setColumns ( strlen ( buf ) );	// set width before instantiation

	snprintf ( help, sizeof ( help ), "Specifies new file: bits per sample, sample rate, channels \
, initial number of zero-value samples; default 16,44100,1,0");
				
	help [ sizeof ( help ) - 1 ] = 0;
	_edit->setHelp ( help );
	_edit->execute();
	_edit->setText ( buf );	// can't set text before instantiation...
	_popupRealised = True;
}


void WaveDefaultsCmd::undoCommand()
{
	// not implemented
}


void WaveDefaultsCmd::enterCallback ( void *clientData, char *text )
{
	WaveDefaultsCmd *obj = ( WaveDefaultsCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->enter ( text );

	obj->_popupRealised = False;
	delete obj->_edit;
}

/* specify resource to be set on 'enter' or 'ok' */
void WaveDefaultsCmd::enter ( char *text )
{
	// check for separator
	int sep = 0;
	
	for ( unsigned int count = 0; count < strlen ( text ); count++ )
		if ( count[text] == ',' )
			sep++;

	if ( sep < 3 )
	{
		char msg[300];
		snprintf ( msg, sizeof ( msg ), "%s invalid: missing comma separator.\n\
Format: bitsPerSample,SamplesPerSec,channels,numSamples", text );
		msg [ sizeof ( msg ) - 1 ] = 0;
		infoDialogManager->post ( msg );
		return;
	}

	int bitsPerSample;
	long samplesPerSec;
	int channels;
	long defaultWavNumSamples;

	sscanf ( text, "%d,%ld,%d,%ld", &bitsPerSample, &samplesPerSec, &channels, &defaultWavNumSamples );

	if ( strlen ( mixwaveResources->csoundCmd() ) != 0 )

	{
		if ( samplesPerSec != mixwaveResources->csoundSampleSize() * mixwaveResources->kRate() )
		{
			char msg[300];
			snprintf ( msg, sizeof ( msg ), "Wave sample rate %d conflicts \
with\nCsound control parameters sRate %d and kRate %d.\nCSound control \
rate (kRate) reset to %d\n", samplesPerSec, mixwaveResources->csoundSampleSize(),
mixwaveResources->kRate(), ( samplesPerSec / mixwaveResources->csoundSampleSize() ) );
			msg [ sizeof ( msg ) - 1 ] = 0;
			infoDialogManager->post ( msg );
			mixwaveResources->kRate ( samplesPerSec / mixwaveResources->csoundSampleSize() );
		}
	}
		
	mixwaveResources->bitsPerSample ( ( short ) bitsPerSample );
	mixwaveResources->samplesPerSec ( samplesPerSec );
	mixwaveResources->channels ( ( short ) channels );
	mixwaveResources->defaultWavNumSamples ( defaultWavNumSamples );

	switch ( bitsPerSample )
	{
		case 8:
			mixwaveResources->minY ( 0 );			// unsigned
			mixwaveResources->maxY ( 255 );
			break;

		case 16:
			mixwaveResources->minY ( -32768 );		// default to signed 16-bit
			mixwaveResources->maxY ( 32767 );
			break;

		case 24:
			mixwaveResources->minY ( -8388608 );	// default to signed 24-bit
			mixwaveResources->maxY ( 8388607 );
			break;		
	}
}


void WaveDefaultsCmd::cancelCallback ( void *clientData )
{
	WaveDefaultsCmd *obj = ( WaveDefaultsCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_edit;
}
